Cuerdo. Arazzo. Context
(cuerdo v0.4.0)
Internal Arazzo Context. Stores workflows and steps inputs/outputs, request/response, etc.
Summary
Functions
Fetch a source description by name. Returns {:ok, source_description, updated_context}
on success, or an error tuple
Creates a new Context, using an existing context as base
Returns the source description name for a given operation path or id
Creates a new context
Same as new/2 but raises on error
Puts a map of %{input_name => value} in the workflow_id inputs. Returns
the updated context
Sets the source description value to the given schema. Returns the updated context
Returns the outputs map for the given stepId, or an empty map if the step does not define any outputs
Returns the outputs map for the given workflowId.
Types
Functions
@spec fetch_source_description(t(), binary()) :: {:ok, map(), t()} | {:error, Exception.t()}
Fetch a source description by name. Returns {:ok, source_description, updated_context}
on success, or an error tuple
@spec from_base(t(), map()) :: {:ok, t()} | {:error, Exception.t()}
Creates a new Context, using an existing context as base
Returns the source description name for a given operation path or id
@spec new(map(), Keyword.t()) :: {:ok, t()} | {:error, Exception.t()}
Creates a new context
Same as new/2 but raises on error
Puts a map of %{input_name => value} in the workflow_id inputs. Returns
the updated context
Sets the source description value to the given schema. Returns the updated context
Returns the outputs map for the given stepId, or an empty map if the step does not define any outputs
Returns the outputs map for the given workflowId.